import pytest
import inspect

from mlair.data_handler.abstract_data_handler import AbstractDataHandler


class TestDefaultDataHandler:

    def test_required_attributes(self):
        dh = AbstractDataHandler
        assert hasattr(dh, "_requirements")
        assert hasattr(dh, "__init__")
        assert hasattr(dh, "build")
        assert hasattr(dh, "requirements")
        assert hasattr(dh, "own_args")
        assert hasattr(dh, "transformation")
        assert hasattr(dh, "get_X")
        assert hasattr(dh, "get_Y")
        assert hasattr(dh, "get_data")
        assert hasattr(dh, "get_coordinates")

    def test_init(self):
        assert isinstance(AbstractDataHandler(), AbstractDataHandler)

    def test_build(self):
        assert isinstance(AbstractDataHandler.build(), AbstractDataHandler)

    def test_requirements(self):
        dh = AbstractDataHandler()
        assert isinstance(dh._requirements, list)
        assert len(dh._requirements) == 0
        assert isinstance(dh.requirements(), list)
        assert len(dh.requirements()) == 0

    def test_own_args(self):
        dh = AbstractDataHandler()
        assert isinstance(dh.own_args(), list)
        assert len(dh.own_args()) == 0
        assert "self" not in dh.own_args()

    def test_transformation(self):
        assert AbstractDataHandler.transformation() is None

    def test_get_X(self):
        dh = AbstractDataHandler()
        with pytest.raises(NotImplementedError):
            dh.get_X()
        assert sorted(["self", "upsampling", "as_numpy"]) == sorted(inspect.getfullargspec(dh.get_X).args)
        assert (False, False) == inspect.getfullargspec(dh.get_X).defaults

    def test_get_Y(self):
        dh = AbstractDataHandler()
        with pytest.raises(NotImplementedError):
            dh.get_Y()
        assert sorted(["self", "upsampling", "as_numpy"]) == sorted(inspect.getfullargspec(dh.get_Y).args)
        assert (False, False) == inspect.getfullargspec(dh.get_Y).defaults

    def test_get_data(self):
        dh = AbstractDataHandler()
        with pytest.raises(NotImplementedError):
            dh.get_data()
        assert sorted(["self", "upsampling", "as_numpy"]) == sorted(inspect.getfullargspec(dh.get_data).args)
        assert (False, False) == inspect.getfullargspec(dh.get_data).defaults

    def test_get_coordinates(self):
        dh = AbstractDataHandler()
        assert dh.get_coordinates() is None
